/*

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.

*/
module derelict.assimp.assimp;

public
{
    import derelict.assimp.types;
    import derelict.assimp.functions;
}

private
{
    import derelict.util.loader;
    import derelict.util.system;

    static if(Derelict_OS_Windows)
    {
        static if (size_t.sizeof == 4)
            enum libNames = "assimp.dll, Assimp32.dll";
        else static if (size_t.sizeof == 8)
            enum libNames = "assimp.dll, Assimp64.dll";
        else
            static assert(0);
    }
    else static if(Derelict_OS_Mac)
        enum libNames = "libassimp.dylib";
    else static if(Derelict_OS_Posix)
        enum libNames = "libassimp.so, libassimp.so.2, libassimp.so.2.0.0";
    else
        static assert(0, "Need to implement ASSIMP libNames for this operating system.");
}

class DerelictASSIMPLoader : SharedLibLoader
{

    protected
    {
        override void loadSymbols()
        {
            bindFunc(cast(void**)&aiImportFile, "aiImportFile");
            bindFunc(cast(void**)&aiImportFileEx, "aiImportFileEx");
            bindFunc(cast(void**)&aiImportFileFromMemory, "aiImportFileFromMemory");
            bindFunc(cast(void**)&aiApplyPostProcessing, "aiApplyPostProcessing");
            bindFunc(cast(void**)&aiGetPredefinedLogStream, "aiGetPredefinedLogStream");
            bindFunc(cast(void**)&aiAttachLogStream, "aiAttachLogStream");
            bindFunc(cast(void**)&aiEnableVerboseLogging, "aiEnableVerboseLogging");
            bindFunc(cast(void**)&aiDetachLogStream, "aiDetachLogStream");
            bindFunc(cast(void**)&aiDetachAllLogStreams, "aiDetachAllLogStreams");
            bindFunc(cast(void**)&aiReleaseImport, "aiReleaseImport");
            bindFunc(cast(void**)&aiGetErrorString, "aiGetErrorString");
            bindFunc(cast(void**)&aiIsExtensionSupported, "aiIsExtensionSupported");
            bindFunc(cast(void**)&aiGetExtensionList, "aiGetExtensionList");
            bindFunc(cast(void**)&aiGetMemoryRequirements, "aiGetMemoryRequirements");
            bindFunc(cast(void**)&aiSetImportPropertyInteger, "aiSetImportPropertyInteger");
            bindFunc(cast(void**)&aiSetImportPropertyFloat, "aiSetImportPropertyFloat");
            bindFunc(cast(void**)&aiSetImportPropertyString, "aiSetImportPropertyString");
            bindFunc(cast(void**)&aiCreateQuaternionFromMatrix, "aiCreateQuaternionFromMatrix");
            bindFunc(cast(void**)&aiDecomposeMatrix, "aiDecomposeMatrix");
            bindFunc(cast(void**)&aiTransposeMatrix4, "aiTransposeMatrix4");
            bindFunc(cast(void**)&aiTransposeMatrix3, "aiTransposeMatrix3");
            bindFunc(cast(void**)&aiTransformVecByMatrix3, "aiTransformVecByMatrix3");
            bindFunc(cast(void**)&aiTransformVecByMatrix4, "aiTransformVecByMatrix4");
            bindFunc(cast(void**)&aiMultiplyMatrix4, "aiMultiplyMatrix4");
            bindFunc(cast(void**)&aiMultiplyMatrix3, "aiMultiplyMatrix3");
            bindFunc(cast(void**)&aiIdentityMatrix3, "aiIdentityMatrix3");
            bindFunc(cast(void**)&aiIdentityMatrix4, "aiIdentityMatrix4");
            bindFunc(cast(void**)&aiGetLegalString, "aiGetLegalString");
            bindFunc(cast(void**)&aiGetVersionMinor, "aiGetVersionMinor");
            bindFunc(cast(void**)&aiGetVersionMajor, "aiGetVersionMajor");
            bindFunc(cast(void**)&aiGetVersionRevision, "aiGetVersionRevision");
            bindFunc(cast(void**)&aiGetCompileFlags, "aiGetCompileFlags");
        }
    }

    public
    {
        this()
        {
            super(libNames);
        }
    }
}

__gshared DerelictASSIMPLoader DerelictASSIMP;

shared static this()
{
    DerelictASSIMP = new DerelictASSIMPLoader();
}

shared static ~this()
{
    DerelictASSIMP.unload();
}
